#include <QtGui>
#include <QDebug>
#include "dialog.h"
#include "ui_dialog.h"

using namespace std;

/**************************************************************************************************
    COPYRIGHT: SHIGERU SUZUKI 2013
	Commercial use of this program its revision is strictly prohibited.
	There is no warranty or guaranty of this program and its results.
	I would appreciate user's report of any bugs or errors of this program.
**************************************************************************************************/

// Meaningful variable names are recommended

const double pi=3.141592;
const double p0=101300.0;           /* absolute atmospheric pressure */
int choice;                          /* input signal selcctor choice option :
									   0: sinsoidal, 1: sweep, other: random input */
int vvso;                           /* velocity vector sum option, 0:all, 1:surface only */

int i,j,k,n,ND,n_cycles,division,n_rows;    /* counter, counter, counter, number of chamber,
											   number of calculation cycles,
											   number of divisons in a term, c_cycles*division */
int req_cycles;                             /* required cycles for DFT analysis */

double Q[25][25],KH[25][25],K[25][25];      /* for recursive formula, standard stiffness matrix,
											   actual stiffness matrix */
double L[25],M[25],A[25],R[25],V[25],force[25]; /* duct length, duct mass, area, area ratio,
											   chamber volume, input force */
double C[25];                                   /* damping coefficients : rev.07 */
double MP[25], MM[25], MK[25][25];      /* modified mass with damping effect */
double xp[25],xc[25],xn[25],sumx;       /* displacement vectors */
double vc[25],sumv;             /* velocity vectors */
double KS[13];                  /* standard stiffness */
double m0,radius,f0,F,phi,b,T,tempo,forte;  /* membranee mass,radius of membrane,f0,
							   input frequency, sweep speed [Hz/s],
							   amplitude of input, time, force */
double zeta;                    /* intermediate variable of damping coefficient */
double Ku, HCR, delta;              /* stiffness of sp unit, thermal condition,
							   time resolution */

FILE *displacement, *displacement_n;        /* raw displacement output & normalized displacement output */
FILE *dft_v;                    /* sum of normalized velocity without time column for DFT */
FILE *velocity, *velocity_n;            /* velocity of each mass & normalized velocity of each mass */
FILE *matrix;                   /* matrices K, RKR, M-1(RKR), Q */


//------------------------------------------------------------------
// CONSTRACTOR
//------------------------------------------------------------------
Dialog::Dialog(QWidget *parent) :
	QDialog(parent),
	ui(new Ui::Dialog)
{
	ui->setupUi(this);

    // GRAPH SETTINGS
	ui->customPlot->setInteractions(QCustomPlot::iRangeDrag | QCustomPlot::iRangeZoom | QCustomPlot::iSelectAxes |
									QCustomPlot::iSelectLegend | QCustomPlot::iSelectPlottables | QCustomPlot::iSelectTitle);
	ui->customPlot->setRangeDrag(Qt::Horizontal|Qt::Vertical);
	ui->customPlot->setRangeZoom(Qt::Horizontal|Qt::Vertical);
	ui->customPlot->xAxis->setRange(0, 8);
	ui->customPlot->yAxis->setRange(-5, 5);
	ui->customPlot->setupFullAxesBox();
    ui->customPlot->setTitle("Velocity of each mass");
    ui->customPlot->xAxis->setLabel("time[s]");
    ui->customPlot->yAxis->setLabel("velocity[m/s]");
	ui->customPlot->legend->setVisible(true);
	QFont legendFont = font();
	legendFont.setPointSize(10);
	ui->customPlot->legend->setFont(legendFont);
	ui->customPlot->legend->setSelectedFont(legendFont);
	ui->customPlot->legend->setSelectable(QCPLegend::spItems); // legend box shall not be selectable, only legend items

    // MOUSE OPERATION IN GRAPH
	connect(ui->customPlot, SIGNAL(mousePress(QMouseEvent*)), this, SLOT(mousePress()));
	connect(ui->customPlot, SIGNAL(mouseWheel(QWheelEvent*)), this, SLOT(mouseWheel()));
	connect(ui->customPlot->xAxis, SIGNAL(rangeChanged(QCPRange)), ui->customPlot->xAxis2, SLOT(setRange(QCPRange)));
	connect(ui->customPlot->yAxis, SIGNAL(rangeChanged(QCPRange)), ui->customPlot->yAxis2, SLOT(setRange(QCPRange)));

    // OBTAIN PARAMETER VALUES
	read_parameter();
    // DRAW GRAPH
	calc();
	plot();
}

//------------------------------------------------------------------
// DESTRUCTOR
//------------------------------------------------------------------
Dialog::~Dialog()
{
	delete ui;
}

void Dialog::mousePress()
{
  // if an axis is selected, only allow the direction of that axis to be dragged
  // if no axis is selected, both directions may be dragged
  
  if (ui->customPlot->xAxis->selected().testFlag(QCPAxis::spAxis))
	ui->customPlot->setRangeDrag(ui->customPlot->xAxis->orientation());
  else if (ui->customPlot->yAxis->selected().testFlag(QCPAxis::spAxis))
	ui->customPlot->setRangeDrag(ui->customPlot->yAxis->orientation());
  else
	ui->customPlot->setRangeDrag(Qt::Horizontal|Qt::Vertical);
}

void Dialog::mouseWheel()
{
  // if an axis is selected, only allow the direction of that axis to be zoomed
  // if no axis is selected, both directions may be zoomed
  
  if (ui->customPlot->xAxis->selected().testFlag(QCPAxis::spAxis))
	ui->customPlot->setRangeZoom(ui->customPlot->xAxis->orientation());
  else if (ui->customPlot->yAxis->selected().testFlag(QCPAxis::spAxis))
	ui->customPlot->setRangeZoom(ui->customPlot->yAxis->orientation());
  else
	ui->customPlot->setRangeZoom(Qt::Horizontal|Qt::Vertical);
}


//------------------------------------------------------------------
// CALCULATION RESULTS
//------------------------------------------------------------------
void Dialog::results()
{
//  THIS FUNCTION SEQUENTIALLY CALCULATE AND WRITE VECTORS

//  WRITE HEADER ROW

//  HEADER FOR RAW VECTOR OUTPUT

	fprintf(displacement,"time[s],force[N]");
	fprintf(velocity,"time[s],force[N]");
	for (i=0; i<ND; i++)
	{
		fprintf(displacement,",x[%d]",i);
		fprintf(velocity,",v[%d]",i);
	}
	if(choice==0)
	{
		fprintf(displacement,",%f[Hz]\n",F);
		fprintf(velocity,",%f[Hz]\n",F);
	}
	else
	{
		fprintf(displacement,",random\n");
		fprintf(velocity,",random\n");
	}

//  HEADER FOR NORMALIZED VECTOR OUTPUT

	fprintf(displacement_n,"time[s],sum_x[mm]");
	fprintf(velocity_n,"time[s],sum_v[m/s]");

	for (i=0; i<ND; i++)
	{
		fprintf(displacement_n,",x_n[%d]",i);
		fprintf(velocity_n,",v_n[%d]",i);
	}
	if(choice==0)                       /* sinusoidal wave */
	{
		fprintf(displacement_n,",force[N],%f[Hz]\n",F);
		fprintf(velocity_n,",force[N],%f[Hz]\n",F);
	}
	else if(choice==1)                  /* sweep signal */
	{
		fprintf(displacement_n,",force[N],sweep rate[Hz/s]=%f,final frequency[Hz]=%f\n",phi,F);
		fprintf(velocity_n,",force[N],sweep rate[Hz/s]=%f,final frequency[Hz]=%f\n",phi,F);
	}
	else                            /* random signal */
	{
		fprintf(displacement_n,",force[N],random\n");
		fprintf(velocity_n,",force[N],random\n");
	}

//  WRITE NUMBER OF POINTS AND DELTA T FOR DFT ANALYSIS

	fprintf(dft_v,"%d\n%f\n",n_rows,delta); /* 1st line: data points, 2nd line: delta t */

//  END OF HEADER PROCESSES

	for (k=0; k<=n_rows; k++)
	{
		tempo=delta*k;                      /* tempo is "time" */

//  SOURCE SIGNAL OPTION (0:SINUSOIDAL, 1:LINEAR SWEEP, 2:RANDOM)

		if(choice==0)
		{
			forte=b*sin(2*pi*F*tempo);          /* sinusoidal signal input */
		}
		else if(choice==1)
		{
			forte=b*sin(2*pi*phi*tempo*tempo);      /* sweep signal input */
		}
		else
		{
			forte=b*2*(rand()%100-50)/100;          /* random signal input */
		}

		force[0]=2*delta*delta/MP[0]*forte;         /* force is converted to recursive formula */


//  VECTOR SUM CALCULATION

		sumx=0;                         /* initial value */
		sumv=0;

		for (j=0; j<ND; j++)
		{
			sumx=sumx+xc[j]*R[j]*1E3;   /* sum of normalized displacement */
		}

//
		for (i=0; i<ND; i++)
		{
			xn[i]=0.0;
			for (j=0; j<ND; j++)
			{
				xn[i]=xn[i]+Q[i][j]*xc[j];
			}
			xn[i]=xn[i]-MM[i]/MP[i]*xp[i]+force[i];
			vc[i]=(xc[i]-xp[i])/delta;
		}

//  CHOOSE IF INTERNAL DUCTS ARE IGNORED

		if(vvso==0)
		{
			vvso=0;                     /* all ducts are used */
		}
		else
		{
			vvso=n;                     /* internal ducts are ignored */
		}

		sumv=sumv+vc[0]*R[0];                   /* velocity of membrane */
		for (j=vvso+1; j<ND; j++)
		{
			sumv=sumv+vc[j]*R[j];   /* velocity of ducts; internal ducts are ignored if vvso=1 */
		}

//  WRITE RAW & NORMALIZED VECTORS

		fprintf(displacement,"%f,%f",tempo,forte);
		fprintf(displacement_n,"%f,%f",tempo,sumx);
		fprintf(velocity,"%f,%f",tempo,forte);
		fprintf(velocity_n,"%f,%f",tempo,sumv);

		for (i=0; i<ND; i++)
		{

			fprintf(displacement,",%f",xc[i]*1E3);      /* raw vector */
			fprintf(displacement_n,",%f",xc[i]*R[i]*1E3);   /* normalized vector */
			fprintf(velocity,",%f",vc[i]);          /* raw vector (m/s)*/
			fprintf(velocity_n,",%f",vc[i]*R[i]);       /* normalized vector (m/s)*/
			xp[i]=xc[i];
			xc[i]=xn[i];

		}

		fprintf(displacement,"\n");
		fprintf(displacement_n,",%f\n",forte);          /* force vector is placed at last column */

		fprintf(velocity,"\n");
		fprintf(velocity_n,",%f\n",forte);          /* force vector is placed at last column */
		fprintf(dft_v,"%f\n",sumv);             /* output for dft analysis */
	}

}



//------------------------------------------------------------------
// MATRICES
//------------------------------------------------------------------
void Dialog::matrices()
{
//  WRITE MATRIX VALUES FOR DEBUGGING PURPOSE ONLY

//  print "i" as header line

	for (i=0; i<ND; i++)
	{
		fprintf(matrix,"%d,",i);
	}
	fprintf(matrix,"\n\n");

//  STANDARD STIFFNESS MATRIX

	fprintf(matrix,"standard stiffness matrix\n");
	for (i=0; i<ND; i++)
	{
		for (j=0; j<ND; j++)
		{
			fprintf(matrix,"%f,",KH[i][j]);
		}
		fprintf(matrix,"\n");
	}
	fprintf(matrix,"\n");

//  ACTUAL STIFFNESS MATRIX

	fprintf(matrix,"actual stiffness matrix\n");

	for (i=0; i<ND; i++)
	{
		for (j=0; j<ND; j++)
		{
			fprintf(matrix,"%f,",K[i][j]);
		}
		fprintf(matrix,"\n");
	}
	fprintf(matrix,"\n");

//  Q MATRIX - INTERMEDIATE MATRIX FOR RECURSIVE CALCULATION -

	fprintf(matrix,"Q matrix\n");
	for (i=0; i<ND; i++)
	{
		for (j=0; j<ND; j++)
		{
			fprintf(matrix,"%f,",Q[i][j]);
		}
		fprintf(matrix,"\n");
	}
}


//------------------------------------------------------------------
// OBTAIN PARAMETER VALUES
//------------------------------------------------------------------
void Dialog::read_parameter()
{
    m0 = ui->m_0->value();
    radius = ui->r_0->value();
    f0 = ui->f_0->value();

    n = ui->n_subchambers->value();
    n_cycles = ui->calc_cycles->value();
    division = ui->resolution->value();
    division=pow(2,division);
    HCR = ui->thermal_condition->value();
    b = ui->force_amplitude->value();

    ND=2*n+1;                       /* number of degrees of freedom */

    V[0] = ui->main_c_volume->value();
    zeta = ui->damp_c_0->value();
    C[0] = 4*pi*zeta*f0*m0/1000;


    if(n==1)
    {
        V[1] = ui->sub_c_volume_1->value();
        A[1] = ui->area_internal_1->value();
        C[1] = ui->damp_c_1->value();
        L[1] = ui->length_internal_1->value();
        A[2] = ui->area_internal_2->value();
        L[2] = ui->length_internal_2->value();
        C[2] = ui->damp_c_2->value();
    }
    if(n==2)
    {
        V[1] = ui->sub_c_volume_1->value();
        A[1] = ui->area_internal_1->value();
        L[1] = ui->length_internal_1->value();
        C[1] = ui->damp_c_1->value();
        A[2] = ui->area_internal_2->value();
        L[2] = ui->length_internal_2->value();
        V[2] = ui->sub_c_volume_2->value();
        C[2] = ui->damp_c_2->value();

        A[3] = ui->area_internal_3->value();
        L[3] = ui->length_internal_3->value();
        C[3] = ui->damp_c_3->value();
        A[4] = ui->area_external_1->value();
        L[4] = ui->length_external_1->value();
        C[4] = ui->damp_c_4->value();
    }
    if(n==3)
    {
        V[1] = ui->sub_c_volume_1->value();
        A[1] = ui->area_internal_1->value();
        L[1] = ui->length_internal_1->value();
        C[1] = ui->damp_c_1->value();
        A[2] = ui->area_internal_2->value();
        L[2] = ui->length_internal_2->value();
        V[2] = ui->sub_c_volume_2->value();
        C[2] = ui->damp_c_2->value();
        V[3] = ui->sub_c_volume_3->value();
        A[3] = ui->area_internal_3->value();
        L[3] = ui->length_internal_3->value();
        C[3] = ui->damp_c_3->value();

        A[4] = ui->area_external_1->value();
        L[4] = ui->length_external_1->value();
        C[4] = ui->damp_c_4->value();
        A[5] = ui->area_external_2->value();
        L[5] = ui->length_external_2->value();
        C[5] = ui->damp_c_5->value();
        A[6] = ui->area_external_3->value();
        L[6] = ui->length_external_3->value();
        C[6] = ui->damp_c_6->value();
    }



//    for (i=0;i<=ND;i++)
//    {
//        C[i]=0.0;
//    }

//    // OPEN FILES
//	FILE *input;                    /* simulation condition */
//	input=fopen("input_parameters.txt","r");        /* input parameter file */
//  READ INPUT PARAMETERS FROM FILE

//	fscanf(input,"%lf\t%lf\t%lf\n",&m0,&radius,&f0);    /* unit's spec */
//	fscanf(input,"%d\t%d\t%d\t%lf\t%lf",&n,&n_cycles,&division,&HCR,&b);

//	ND=2*n+1;                       /* number of degrees of freedom */

//  READ REMAINING INPUT PARAMETERS FROM FILE

//	for (i=0; i<=n; i++)
//	{
//		fscanf(input,"%lf\t",&V[i]);   /* volume of chambers */
//	}
//	for (i=1; i<ND; i++)
//	{
//		fscanf(input,"%lf\t",&A[i]);   /* area of ducts */
//	}
//	for (i=1; i<ND; i++)
//	{
//		fscanf(input,"%lf\t",&L[i]);   /* length of ducts */
//	}
//	for (i=0; i<ND; i++)
//	{
//		fscanf(input,"%lf\t",&C[i]);   /* damping coefficients */
//	}
//	/* n, cycles, heat capacity ratio, force */
//	fclose(input);
}


//------------------------------------------------------------------
// Calc Button Clicked
//------------------------------------------------------------------
void Dialog::on_pushButton_clicked()
{
    // OBTAIN PARAMETER VALUES
	read_parameter();
	calc();
	csv_output();
}

//------------------------------------------------------------------
// CSV OUTPUT
//------------------------------------------------------------------
void Dialog::csv_output()
{
	displacement=fopen("x_vectors.csv","w");        /* raw displacement vector */
	displacement_n=fopen("normalized_x_vectors.csv","w");   /* normalized displacement vector */

	velocity=fopen("v_vectors.csv","w");            /* raw velocity vector */
	velocity_n=fopen("normalized_v_vectors.csv","w");   /* normalized velocity vector */
	dft_v=fopen("dftv.txt","w");                /* velocity output for DFT */

	matrix=fopen("matrices.csv","w");           /* stiffness and other matrices */

    QString msg;
	msg.sprintf("Ready to start recursive calculation!");
	ui->textEdit->append(msg);

	results();  /* calculate recursive formulae and save in a file */

	msg.sprintf("Dynamic simulation has been complete!");
	ui->textEdit->append(msg);

	matrices(); /* save matrix files for debugging */
	msg.sprintf("Matrces have been recorded.");
	ui->textEdit->append(msg);

	fclose(displacement);
	fclose(displacement_n);
	fclose(velocity);
	fclose(velocity_n);
	fclose(matrix);
	fclose(dft_v);

}


//------------------------------------------------------------------
// SLIDER MOVED
//------------------------------------------------------------------
void Dialog::on_freqSlider_valueChanged(int value)
{
    // CHANGE CONSTANT FREQUENCY
	ui->dblFrequency->setValue((double)value);
}

//------------------------------------------------------------------
// FREQUENCY CHANGED
//------------------------------------------------------------------
void Dialog::on_dblFrequency_valueChanged(double arg1)
{
    // REDRAW PLOT
    redraw();
    // SLIDE OPERATED
	ui->freqSlider->setValue((int)arg1);
}

//------------------------------------------------------------------
// Redraw Plot Button
//------------------------------------------------------------------
void Dialog::on_pushButton_2_clicked()
{
    // REDRAW PLOT
    redraw();
}

//------------------------------------------------------------------
// GRAPH REDRAW
//------------------------------------------------------------------
void Dialog::redraw()
{
    // OBTAIN PARAMETER VALUES
    read_parameter();
    // DRAW PLOT
    calc();
    plot();
}

//------------------------------------------------------------------
// CALCULATION ONLY
//------------------------------------------------------------------
void Dialog::calc()
{
//  INTERMEDIATE PARAMETER VALUES

	A[0]=pi*pow(radius,2);                  /* area of membrane */
	M[0]=m0*1.0E-3;                     /* mass of membrane unit conversion to [kg] */
	L[0]=m0*1.0E-3/A[0]*10000.0/1.2*1000;           /* not used anywhere but put into memory */
	R[0]=1.0;                       /* A[0]/A[0]=1 */
	Ku=4.0*pow(pi,2)*pow(f0,2)*M[0];            /* spring constant of sp unit */

	QString msg = QString("[0] values and parameters caluculation done!");
	ui->textEdit->append(msg);

//  CHOOSE USING ALL DUCT OR IGNORING INTERNAL DUCTS FOR VELOCITY VECTOR SUM CALCULATION

	vvso = 0;
	if(ui->radioIgnore->isChecked())
		vvso = 1;

//  CHOOSE INPUT SIGNAL OPTIONS

	choice = 0;
	if(ui->radioLinearSweep->isChecked())
		choice = 1;
	else if(ui->radioRandom->isChecked())
		choice = 2;

	if(choice==0)
	{
		F = ui->dblFrequency->value();
		req_cycles=F;               /* compute required cycles */
		n_rows=n_cycles*division;       /* number of data rows */
		T=1/F;                  /* term of wave in [s]*/
		delta=T/division;           /* time resolution */

		msg.sprintf("You have chosen sinsoidal wave at %f [Hz].", F);
		ui->textEdit->append(msg);
	}
	else if(choice==1)              /* processing sweep signal */
	{
		F = ui->dblFrequency->value();
		phi = ui->dblSweepRate->value();

		if(phi<10)
		{
			phi=10;   /* 1[Hz/s] <= sweep speed <=200[Hz/s] */
		}
		if(phi>200)
		{
			phi=200;
		}

		T=1/F;                  /* term of wave in [s]*/
		delta=T/division;           /* time resolution */

		if(F>1000)
		{
			F=1000;   /* limit maxmum frequency is 1000[Hz] */
		}
		if(F<100)
		{
			F=100;   /* at least sweep to 100[Hz] */
		}

        n_rows= (int)(5*F/phi/delta);     /* update number of cycles to cover max frequency */

		msg.sprintf("Number of Rows = %d",n_rows);
		ui->textEdit->append(msg);
		msg.sprintf("You have chosen linear sweep at %f [Hz/s] up to %f [Hz].",phi,F);
		ui->textEdit->append(msg);
	}
	else                        /* wrong values are assumed random signal was chosen */
	{
		F=100;                  /* 100Hz is assumed for random signal analysis
							time resolution = 1/100/division for random */
		req_cycles=F;               /* assuming 100Hz signal */
		n_cycles=req_cycles;
		n_rows=n_cycles*division;       /* number of data rows */
		T=1/F;                  /* term of wave in [s]*/
		delta=T/division;           /* time resolution */

		msg.sprintf("You have chosen random signal.");
		ui->textEdit->append(msg);
	}

//  chamber volume unit conversion litre to m^3

	for (i=0; i<=n; i++)
	{
		V[i]=V[i]/1000.0;
	}

	msg.sprintf("Unit conversion of chamber volume is complete!");
	ui->textEdit->append(msg);

	for (i=1; i<ND; i++)
	{
		R[i]=A[i]/A[0];                     /* ratio of areas without unit conversion */
		M[i]=A[i]*1.0E-4*L[i]*1.0E-3*1.2;           /* mass vector calculation [kg] */
	}

	for (i=0; i<ND; i++)
	{
		MP[i]=2*M[i]+delta*C[i];            /* modified mass matrix for damping compensation */
		MM[i]=2*M[i]-delta*C[i];            /* modified mass matrix for damping compensation */
	}

// check only
	msg.sprintf("i\tR[i]\tA[i]\tL[i]\tM[i]");
	ui->textEdit->append(msg);
	for (i=0; i<ND; i++)
	{
		msg.sprintf("%d\t%f\t%f\t%f\t%f",i,R[i],A[i],L[i],M[i]);
		ui->textEdit->append(msg);
	}
	msg.sprintf("\n-- damping effects below --");
	ui->textEdit->append(msg);
	QString str = "";
	for (i=0; i<ND; i++)
	{
		msg.sprintf("%f\t",(1-MM[i]/MP[i]));
		str += msg;
	}
	ui->textEdit->append(str);

	msg.sprintf("Ratio and mass vector calculation is complete!");
	ui->textEdit->append(msg);

	for (i=0; i<=n; i++)
	{
		KS[i]= HCR*A[0]*A[0]*1E-8*p0/V[i];
		msg.sprintf("Standard stiffness matrix values %d\t%f",i,KS[i]);
		ui->textEdit->append(msg);
	}

//  initial value setting to avoid bad values
	for (i=0; i<25; i++)
	{
		xp[i]=0.0;      /* previous displacement vector */
		xc[i]=0.0;      /* current displacement vector */
		xn[i]=0.0;      /* next displacement vector */
		force[i]=0.0;       /* force vector */
//
		for (j=0; j<25; j++)
		{
			KH[i][j]=0.0;       /* standard stiffness matrix */
			K[i][j]=0.0;        /* actual stiffness matrix */
			Q[i][j]=0.0;        /* intermediate matrix for calculation */
		}
	}
//  begin setting standard stiffness matrix

	for (i=0; i<=n; i++)
	{
		for (j=0; j<=n; j++)
		{
			KH[i][j]=KS[0];     /* upper left block */
		}
	}
	msg.sprintf("Upper left block of standard stiffness matrix values are calculated!");
	ui->textEdit->append(msg);

	KH[0][0]=KH[0][0]+Ku;
	for (i=1; i<=n; i++)
	{
		KH[i][i]=KH[i][i]+KS[i];    /* diagonal element of upper left block */
	}
	for (i=1; i<=n; i++)
	{
		KH[n+i][i]=-KS[i];  /* lower left block */
		KH[i][n+i]=-KS[i];  /* upper right block */
	}
	for (i=1; i<=n; i++)
	{
		KH[n+i][n+i]=KS[i]; /* lower right block */
	}
//  end setting standard stiffness matrix

	msg.sprintf("Standard stiffnes matrix became ready!");
	ui->textEdit->append(msg);

//  begin calculating actual stiffness matrix
	for (i=0; i<ND; i++)
	{
		for (j=0; j<ND; j++)
		{
			K[i][j]=KH[i][j]*R[i]*R[j];
		}
	}
//  stiffness matrix complete
	msg.sprintf("Actual stiffnes matrix became ready!");
	ui->textEdit->append(msg);


//  MK[i][j]=2*(2*M[i]-delta^2*K[i][j]) rev07 and later
//   -- this matrix is to calculate coefficient matrix of x[j] --

	for(i=0; i<ND; i++)
	{
		for(j=0; j<ND; j++)
		{
			MK[i][j]=0.0;   /* initial value = 0 */
		}
	}

	for(i=0; i<ND; i++)
	{
		MK[i][i]=M[i];                  /* diagonal elements are given */
	}
	for(i=0; i<ND; i++)
	{
		for(j=0; j<ND; j++)
		{
			MK[i][j]=2*(2*MK[i][j]-pow(delta,2)*K[i][j]); /* all elements */
		}
	}

//  Calculate "Q=MK/MP" rev07 and later
//   -- this is coefficient matrix of x[j] --

	for (i=0; i<ND; i++)
	{
		for (j=0; j<ND; j++)
		{
			Q[i][j]=MK[i][j]/MP[i];
		}
	}
}

//------------------------------------------------------------------
// GRAPH DRAW
//------------------------------------------------------------------
void Dialog::plot()
{
    // CHANGE MOUSE CURSOR WITH WATCH
	QApplication::setOverrideCursor(Qt::WaitCursor);
	QString msg;

	msg.sprintf("Header rows were written!");          /* notice only */
	ui->textEdit->append(msg);
	msg.sprintf("Now recursive calculation process begins.");
	ui->textEdit->append(msg);

//  RECURSIVE STEP CALCULATION BEGINS

	QVector<double> cx    (n_rows+1);
	QVector<double> ctime (n_rows+1);
	QVector<double> csumv (n_rows+1);
	QVector<double> cv_n0 (n_rows+1);
	QVector<double> cv_n1 (n_rows+1);
	QVector<double> cv_n2 (n_rows+1);
	QVector<double> cv_n3 (n_rows+1);
	QVector<double> cv_n4 (n_rows+1);
	QVector<double> cv_n5 (n_rows+1);
	QVector<double> cv_n6 (n_rows+1);
	QVector<double> cforce(n_rows+1);

	double cmin = 0.0;
	double cmax = 0.0;
    double plot_range;
	for (k=0; k<=n_rows; k++)
	{
//		cx[k] = k;      /* Sr. Osako's original */
        tempo=delta*k;  /* tempo is "time" */
        cx[k]=tempo;     /* modified by SS */

//  SOURCE SIGNAL OPTION (0:SINUSOIDAL, 1:LINEAR SWEEP, 2:RANDOM)

		if(choice==0)
		{
            forte=b*sin(2*pi*F*tempo);      /* sinusoidal signal input */
		}
		else if(choice==1)
		{
			forte=b*sin(2*pi*phi*tempo*tempo);      /* sweep signal input */
		}
		else
		{
			forte=b*2*(rand()%100-50)/100;          /* random signal input */
		}

		force[0]=2*delta*delta/MP[0]*forte;         /* force is converted to recursive formula */


//  VECTOR SUM CALCULATION

		sumx=0;                         /* initial value */
		sumv=0;

		for (j=0; j<ND; j++)
		{
			sumx=sumx+xc[j]*R[j]*1E3;   /* sum of normalized displacement */
		}

//
		for (i=0; i<ND; i++)
		{
			xn[i]=0.0;
			for (j=0; j<ND; j++)
			{
				xn[i]=xn[i]+Q[i][j]*xc[j];
			}
			xn[i]=xn[i]-MM[i]/MP[i]*xp[i]+force[i];
			vc[i]=(xc[i]-xp[i])/delta;
		}

//  CHOOSE IF INTERNAL DUCTS ARE IGNORED

		if(vvso==0)
		{
			vvso=0;                     /* all ducts are used */
		}
		else
		{
			vvso=n;                     /* internal ducts are ignored */
		}

		sumv=sumv+vc[0]*R[0];                   /* velocity of membrane */
		for (j=vvso+1; j<ND; j++)
		{
			sumv=sumv+vc[j]*R[j];   /* velocity of ducts; internal ducts are ignored if vvso=1 */
		}

//  WRITE RAW & NORMALIZED VECTORS

		ctime[k] = tempo;
		csumv[k] = sumv;

		for (i=0; i<ND; i++)
		{
			double dval = vc[i] * R[i];
			switch(i){
				case 0: cv_n0[k] = dval; break;
				case 1: cv_n1[k] = dval; break;
				case 2: cv_n2[k] = dval; break;
				case 3: cv_n3[k] = dval; break;
				case 4: cv_n4[k] = dval; break;
				case 5: cv_n5[k] = dval; break;
				case 6: cv_n6[k] = dval; break;
			}
			cmin = (cmin > dval)? dval:cmin;
			cmax = (cmax < dval)? dval:cmax;
			xp[i]=xc[i];
			xc[i]=xn[i];
		}
		cforce[k] = forte;
	}

    // PLOT AREA
	ui->customPlot->clearGraphs();
//    ui->customPlot->xAxis->setRange(0, n_rows+80);  // xAxis need to be replaced with real time
    ui->customPlot->xAxis->setRange(0, delta*(n_rows+80));  /* modified by SS */
//	ui->customPlot->yAxis->setRange(cmin-0.01, cmax+0.01);
    plot_range = ui->PlotRange->value();
    ui->customPlot->yAxis->setRange(-plot_range, plot_range);

    // PEN DECLARATION
	QPen graphPen;
    graphPen.setWidthF(1);      // PEN SIZE

//  ui->customPlot->addGraph();
//  graphPen.setColor(Qt::red); // COLOUR
//  ui->customPlot->graph()->setPen(graphPen);
//  ui->customPlot->graph()->setName("times");
//  ui->customPlot->graph()->setData(cx, ctime);
//  ui->customPlot->replot();

    if(ui->chkSumv->isChecked() )
    {
		ui->customPlot->addGraph();
        graphPen.setColor(Qt::blue);    // COLOUR
		ui->customPlot->graph()->setPen(graphPen);
		ui->customPlot->graph()->setName("sumv");
		ui->customPlot->graph()->setData(cx, csumv);
		ui->customPlot->replot();
	}
    if(ui->chkMembrane->isChecked() )
    {
        ui->customPlot->addGraph();
        graphPen.setColor(Qt::darkCyan);    // COLOUR
        ui->customPlot->graph()->setPen(graphPen);
        ui->customPlot->graph()->setName("Membrane");
        ui->customPlot->graph()->setData(cx, cv_n0);
        ui->customPlot->replot();
    }
    if(ui->chkDuct1->isChecked() )
    {
        ui->customPlot->addGraph();
        graphPen.setColor(Qt::darkGreen);   // COLOUR
        ui->customPlot->graph()->setPen(graphPen);
        ui->customPlot->graph()->setName("Duct1");
        ui->customPlot->graph()->setData(cx, cv_n1);
        ui->customPlot->replot();
    }
    if(ui->chkDuct2->isChecked() )
    {
        ui->customPlot->addGraph();
        graphPen.setColor(Qt::magenta); // COLOUR
        ui->customPlot->graph()->setPen(graphPen);
        ui->customPlot->graph()->setName("Duct2");
        ui->customPlot->graph()->setData(cx, cv_n2);
        ui->customPlot->replot();
    }
    if(ui->chkDuct3->isChecked() )
    {
        ui->customPlot->addGraph();
        graphPen.setColor(Qt::darkMagenta); // COLOUR
        ui->customPlot->graph()->setPen(graphPen);
        ui->customPlot->graph()->setName("Duct3");
        ui->customPlot->graph()->setData(cx, cv_n3);
        ui->customPlot->replot();
    }
    if(ui->chkDuct4->isChecked() )
    {
        ui->customPlot->addGraph();
        graphPen.setColor(Qt::darkGray);    // COLOUR
        ui->customPlot->graph()->setPen(graphPen);
        ui->customPlot->graph()->setName("Duct4");
        ui->customPlot->graph()->setData(cx, cv_n4);
        ui->customPlot->replot();
    }
    if(ui->chkDuct5->isChecked() )
    {
        ui->customPlot->addGraph();
        graphPen.setColor(Qt::black);   // COLOUR
        ui->customPlot->graph()->setPen(graphPen);
        ui->customPlot->graph()->setName("Duct5");
        ui->customPlot->graph()->setData(cx, cv_n5);
        ui->customPlot->replot();
    }
    if(ui->chkDuct6->isChecked() )
    {
        ui->customPlot->addGraph();
        graphPen.setColor(Qt::darkRed); // COLOUR
        ui->customPlot->graph()->setPen(graphPen);
        ui->customPlot->graph()->setName("Duct6");
        ui->customPlot->graph()->setData(cx, cv_n6);
        ui->customPlot->replot();
    }
	if( ui->chkForce->isChecked() ){
		ui->customPlot->addGraph();
        graphPen.setColor(Qt::darkYellow);  // COLOUR
		ui->customPlot->graph()->setPen(graphPen);
		ui->customPlot->graph()->setName("force");
		ui->customPlot->graph()->setData(cx, cforce);
		ui->customPlot->replot();
	}
    // RESTORE MOUSE CURSUR AS ORIGINAL FROM WATCH
	QApplication::restoreOverrideCursor();
}


//------------------------------------------------------------------
// RADIO BUTTON CLICKED
//------------------------------------------------------------------
void Dialog::on_radioSinusoidal_clicked()
{
    // GRAPH REDRAW
    redraw();
}

void Dialog::on_radioLinearSweep_clicked()
{
    // GRAPH REDRAW
    redraw();

}

void Dialog::on_radioRandom_clicked()
{
    // GRAPH REDRAW
    redraw();
}
